; PROCEDURE COMMENTS +
; NAME: PRO4SAIL5B
; AUTHOR: Translated from Matlab to IDL by Eben N. Broadbent 
;  Dept. of Biology, Stanford University.
;  
; Original Matlab code written by: Jean-Baptiste Féret (feret@ipgp.fr)
; Institut de Physique du Globe de Paris, Space and Planetary Geophysics
; during October 2009 based on a version of PROSAIL provided by
; Wout Verhoef, NLR on April/May 2003.
; 
; CONTACT INFO: ebennb@gmail.com
; DESCRIPTION:
; CALLING SEQUENCE:
;       PRO4SAIL5B,Cab,Car,Cbrown,Cw,Cm,Ns,angl,lai,hspot,tts,tto,psi,psoil,skyl,resh,resv
; INPUTS: Cab,Car,Cbrown,Cw,Cm,Ns,angl,lai,hspot,tts,tto,psi,psoil,skyl
; OUTPUTS: resh,resv
; OPTIONAL OUTPUTS:
; OPTIONAL INPUT KEYWORD(S):
; NOTES:
; METHOD:
; EXAMPLE:
; MODIFICATION HISTORY:
;       Downloaded from "http://teledetection.ipgp.jussieu.fr/prosail/" on 09/10/2010
;       Translated from matlab to IDL by: Eben N. Broadbent on '09/27/2010'
; CODING NOTES:
; Same as Matlab (09/24/2010)
;
;-

pro PRO4SAIL5B,Cab,Car,Cbrown,Cw,Cm,Ns,angl,lai,hspot,tts,tto,psi,psoil,skyl,resh,resv

 data=dataSpec_P5B();;
 Es=data(7,*);;
 Ed=data(8,*);;
 Rsoil1=data(9,*);;
 Rsoil2=data(10,*);;

 ; LEAF OPTICAL PROPERTIES
 prospect_5B,Cab,Car,Cbrown,Cw,Cm,Ns,l,RN,TN;;                              
 rho = RN;;                           
 tau = TN;;                           
                                                           
 ; LEAF ANGLE DISTRIBUTION                           
 na     = 18;;
 rd     = !pi/180.0;;
 lna    = linspace(0,17,18);; 
 thetal = 2.5+5.0*lna;;
 lidf   = calc_LIDF(angl);;                                                          
                                                             
 ; Direct (Es) / Diffuse (Ed) light   
 PARdiro = (1.0-skyl/100.0)*Es;;                                             
 PARdifo = (skyl/100.0)*Ed;;                                                 
                                                        
 ; Soil Reflectance Properties                           
 ; rsoil1 = dry soil                             
 ; rsoil2 = wet soil                             
 rsoil0=psoil*Rsoil1+(1.0-psoil)*Rsoil2;;                 

 cts    = cos(rd*tts);
 cto    = cos(rd*tto);
 ctscto = cts*cto;
 tants  = tan(rd*tts);
 tanto  = tan(rd*tto);
 cospsi = cos(rd*psi);
 dso    = sqrt(tants*tants+tanto*tanto-2.0*tants*tanto*cospsi);

 ; angular distance, compensation of shadow length
 ; Calculate geometric factors associated with extinction and scattering 
 ; Initialise sums
 ks  = 0.0;
 ko  = 0.0;
 bf  = 0.0;
 sob = 0.0;
 sof = 0.0;

 ; Weighted sums over LIDF
 for i=0, na-1 do begin;;                
  ttl = thetal(i);;   leaf inclination discrete values
  ctl = cos(rd*ttl);;
  
  ; SAIL volume scattering phase function gives interception and portions to be 
  ; multiplied by rho and tau
  
  volscatt,tts,tto,psi,ttl,chi_s,chi_o,frho,ftau;; (for a small subset of values looks good, need more thorough verification)
  
  ;********************************************************************************
  ;*                   SUITS SYSTEM COEFFICIENTS 
  ;*
  ;*  ks  : Extinction coefficient for direct solar flux
  ;*  ko  : Extinction coefficient for direct observed flux
  ;*  att : Attenuation coefficient for diffuse flux
  ;*  sigb : Backscattering coefficient of the diffuse downward flux
  ;*  sigf : Forwardscattering coefficient of the diffuse upward flux
  ;*  sf  : Scattering coefficient of the direct solar flux for downward diffuse flux
  ;*  sb  : Scattering coefficient of the direct solar flux for upward diffuse flux
  ;*  vf   : Scattering coefficient of upward diffuse flux in the observed direction
  ;*  vb   : Scattering coefficient of downward diffuse flux in the observed direction
  ;*  w   : Bidirectional scattering coefficient
  ;********************************************************************************

  ; Extinction coefficients
  ksli = chi_s/cts;;
  koli = chi_o/cto;;

  ; Area scattering coefficient fractions
  sobli = frho*!pi/ctscto;;
  sofli = ftau*!pi/ctscto;;
  bfli  = ctl*ctl;;
  ks    = ks+ksli*lidf(i);;
  ko    = ko+koli*lidf(i);;
  bf    = bf+bfli*lidf(i);;
  sob   = sob+sobli*lidf(i);;
  sof   = sof+sofli*lidf(i);;
 endfor;;
 
 ; Geometric factors to be used later with rho and tau
 sdb = 0.5*(ks+bf);
 sdf = 0.5*(ks-bf);
 dob = 0.5*(ko+bf);
 dof = 0.5*(ko-bf);
 ddb = 0.5*(1.0+bf);
 ddf = 0.5*(1.0-bf);

 ; Here rho and tau come in
 sigb= ddb*rho+ddf*tau;
 sigf= ddf*rho+ddb*tau;
 att = 1.0-sigf;
 m2  =(att+sigb)*(att-sigb);

 m2_index = where(m2 le 0.0, m2_count)
 if (m2_count gt 0.0) then m2(m2_index) = 0.0 

 m=sqrt(m2);

 sb  = sdb*rho+sdf*tau;
 sf  = sdf*rho+sdb*tau;
 vb  = dob*rho+dof*tau;
 vf  = dof*rho+dob*tau;
 w = sob*rho+sof*tau;

; Outputs for the case LAI = 0
 if (lai lt 0) then begin;
  tss    = 1.0;
  too    = 1.0;
  tsstoo = 1.0;
  rdd    = 0.0;
  tdd    = 1.0;
  rsd    = 0.0;
  tsd    = 0.0;
  rdo    = 0.0;
  tdo    = 0.0;
  rso    = 0.0;
  rsos   = 0.0;
  rsod   = 0.0;
  rddt   = rsoil0;
  rsdt   = rsoil0;
  rdot   = rsoil0;
  rsodt  = 0.0;
  rsost  = rsoil0;
  rsot   = rsoil0;
  return
 endif;

 ; Other cases (LAI > 0)
 e1    = exp(-m*lai);
 e2    = e1*e1;
 rinf  = (att-m)/sigb;
 rinf2 = rinf*rinf;
 re    = rinf*e1;
 denom = 1.0-rinf2*e2;

 J1ks=Jfunc1(ks,m,lai);
 J2ks=Jfunc2(ks,m,lai);
 J1ko=Jfunc1(ko,m,lai);
 J2ko=Jfunc2(ko,m,lai);

 Ps = (sf+sb*rinf)*J1ks;
 Qs = (sf*rinf+sb)*J2ks;
 Pv = (vf+vb*rinf)*J1ko;
 Qv = (vf*rinf+vb)*J2ko;

 rdd = rinf*(1.0-e2)/denom;
 tdd = (1.0-rinf2)*e1/denom;
 tsd = (Ps-re*Qs)/denom;
 rsd = (Qs-re*Ps)/denom;
 tdo = (Pv-re*Qv)/denom;
 rdo = (Qv-re*Pv)/denom;

 tss = exp(-ks*lai);
 too = exp(-ko*lai);
 z = Jfunc3(ks,ko,lai);
 g1  = (z-J1ks*too)/(ko+m);
 g2  = (z-J1ko*tss)/(ks+m);
    
 Tv1 = (vf*rinf+vb)*g1;
 Tv2 = (vf+vb*rinf)*g2;
 T1  = Tv1*(sf+sb*rinf);
 T2  = Tv2*(sf*rinf+sb);
 T3  = (rdo*Qs+tdo*Ps)*rinf;

 ; Multiple scattering contribution to bidirectional canopy reflectance
 rsod = (T1+T2-T3)/(1.0-rinf2);
 ; Treatment of the hotspot-effect
 alf=1e6;
 ; Apply correction 2/(K+k) suggested by F.-M. Bréon
 if hspot gt 0.0 then alf=(dso/hspot)*2.0/(ks+ko);
 if alf gt 200.0 then alf=200.0;
 if alf eq 0.0 then begin;
  ; The pure hotspot - no shadow
  tsstoo = tss;
  sumint = (1.0-tss)/(ks*lai);
 endif else begin;
  ; Outside the hotspot
  fhot=lai*sqrt(ko*ks);
  ; Integrate by exponential Simpson method in 20 steps
  ; the steps are arranged according to equal partitioning
  ; of the slope of the joint probability function
  x1=0.0;
  y1=0.0;
  f1=1.0;
  fint=(1.0-exp(-alf))*0.05;
  sumint=0;
  
  for i=1,20 do begin;
   if i lt 20 then begin;
    x2=-alog(1.0-i*fint)/alf;
   endif else begin;
    x2=1;
   endelse;
   y2=-(ko+ks)*lai*x2+fhot*(1.0-exp(-alf*x2))/alf;
   f2=exp(y2);
   sumint=sumint+(f2-f1)*(x2-x1)/(y2-y1);
   x1=x2;
   y1=y2;
   f1=f2;
  endfor;
  tsstoo=f1;
 endelse;

; Bidirectional reflectance
 rsos = w*lai*sumint;         Single scattering contribution
 rso=rsos+rsod;               Total canopy contribution
 dn=1.0-rsoil0*rdd;           Interaction with the soil

 rddt=rdd+tdd*rsoil0*tdd/dn;
 rsdt=rsd+(tsd+tss)*rsoil0*tdd/dn;
 rdot=rdo+tdd*rsoil0*(tdo+too)/dn;

 rsodt=rsod+((tss+tsd)*tdo+(tsd+tss*rsoil0*rdd)*too)*rsoil0/dn;
 rsost=rsos+tsstoo*rsoil0;
 rsot=rsost+rsodt;

 resh  = (rddt*PARdifo+rsdt*PARdiro)/(PARdiro+PARdifo);
 resv  = (rdot*PARdifo+rsot*PARdiro)/(PARdiro+PARdifo);
 
end;